﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Input
Imports System.Windows.Media.Animation
Imports Microsoft.Phone.Controls

Namespace MiddlemarchReader
    Partial Public Class BookViewer
        Inherits UserControl
        Private pageHosts(2) As Border
        Private pageHostBaseIndex As Integer = 0 ' current page
        Private dragFactor As Double

        Private nextPageChapterIndex As Integer
        Private nextPagePageIndex As Integer
        Private prevPageChapterIndex As Integer
        Private prevPagePageIndex As Integer

        Private animationInProgress, gestureInProgress, gestureHadFlick, canGoNextPage, canGoPrevPage As Boolean

        Public Shared ReadOnly PageProviderProperty As DependencyProperty = DependencyProperty.Register("PageProvider", GetType(IPageProvider), GetType(BookViewer), New PropertyMetadata(AddressOf OnPageProviderChanged))

        Public Shared ReadOnly PageTransitionProperty As DependencyProperty = DependencyProperty.Register("PageTransition", GetType(PageTransition), GetType(BookViewer), New PropertyMetadata(AddressOf OnPageTransitionChanged))

        Public Shared ReadOnly ChapterIndexProperty As DependencyProperty = DependencyProperty.Register("ChapterIndex", GetType(Integer), GetType(BookViewer), New PropertyMetadata(0, AddressOf OnPageChanged))

        Public Shared ReadOnly PageIndexProperty As DependencyProperty = DependencyProperty.Register("PageIndex", GetType(Integer), GetType(BookViewer), New PropertyMetadata(0, AddressOf OnPageChanged))

        Public Event PageChanged As EventHandler

        Public Sub New()
            InitializeComponent()

            pageHosts(0) = pageHost0
            pageHosts(1) = pageHost1
            pageHosts(2) = pageHost2
        End Sub

        Public Property PageProvider() As IPageProvider
            Set(ByVal value As IPageProvider)
                SetValue(PageProviderProperty, value)
            End Set
            Get
                Return CType(GetValue(PageProviderProperty), IPageProvider)
            End Get
        End Property

        Public Property PageTransition() As PageTransition
            Set(ByVal value As PageTransition)
                SetValue(PageTransitionProperty, value)
            End Set
            Get
                Return CType(GetValue(PageTransitionProperty), PageTransition)
            End Get
        End Property

        Public Property ChapterIndex() As Integer
            Protected Set(ByVal value As Integer)
                SetValue(ChapterIndexProperty, value)
            End Set
            Get
                Return CInt(GetValue(ChapterIndexProperty))
            End Get
        End Property

        Public Property PageIndex() As Integer
            Protected Set(ByVal value As Integer)
                SetValue(PageIndexProperty, value)
            End Set
            Get
                Return CInt(GetValue(PageIndexProperty))
            End Get
        End Property

        Public Function JumpToPage(ByVal chapterIndex As Integer, ByVal pageIndex As Integer) As Boolean
            ' Save chapter and page indices in public properties
            Me.ChapterIndex = chapterIndex
            Me.PageIndex = pageIndex

            If Me.PageProvider IsNot Nothing Then
                Me.PageProvider.SetPageSize(New Size(pageHost0.ActualWidth, pageHost0.ActualHeight))
                Me.PageProvider.SetFont(Me.FontFamily, Me.FontSize)
                Dim element As FrameworkElement = Me.PageProvider.GetPage(chapterIndex, pageIndex)

                If element IsNot Nothing Then
                    ' Set pageHosts for this page and the next page
                    pageHosts(pageHostBaseIndex).Child = element
                    SetUpNextPage()
                    SetUpPreviousPage()

                    ' Set the FractionlBaseIndex for the PageTransition
                    Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex
                    Return True
                End If
            End If
            Return False
        End Function

        Private Sub OnGestureListenerGestureBegin(ByVal sender As Object, ByVal args As GestureEventArgs)
            gestureInProgress = Not animationInProgress

            If gestureInProgress Then
                gestureHadFlick = False
                canGoNextPage = pageHosts((pageHostBaseIndex + 1) Mod 3).Child IsNot Nothing
                canGoPrevPage = pageHosts((pageHostBaseIndex + 2) Mod 3).Child IsNot Nothing
            End If
        End Sub

        Private Sub OnGestureListenerGestureCompleted(ByVal sender As Object, ByVal args As GestureEventArgs)
            gestureInProgress = False
        End Sub

        Private Sub OnGestureListenerTap(ByVal sender As Object, ByVal args As GestureEventArgs)
            If gestureInProgress AndAlso canGoNextPage Then
                AnimateToNextPage()
            End If
        End Sub

        Private Sub OnGestureListenerFlick(ByVal sender As Object, ByVal args As FlickGestureEventArgs)
            If gestureInProgress Then
                If args.Angle < 45 OrElse args.Angle > 315 Then
                    If Not canGoPrevPage Then
                        If Me.ChapterIndex <> 0 AndAlso Me.PageIndex = 0 Then
                            pageHosts((pageHostBaseIndex + 2) Mod 3).Child = PageProvider.GetLastPage(prevPageChapterIndex, prevPagePageIndex)
                            canGoPrevPage = pageHosts((pageHostBaseIndex + 2) Mod 3).Child IsNot Nothing
                        End If
                    End If
                    If canGoPrevPage Then
                        gestureHadFlick = True
                        AnimateToPreviousPage()
                    End If

                ElseIf args.Angle > 135 AndAlso args.Angle < 225 AndAlso canGoNextPage Then
                    gestureHadFlick = True
                    AnimateToNextPage()
                End If
            End If
        End Sub

        Private Sub OnGestureListenerDragStarted(ByVal sender As Object, ByVal args As DragStartedGestureEventArgs)
            dragFactor = 0
        End Sub

        Private Sub OnGestureListenerDragDelta(ByVal sender As Object, ByVal args As DragDeltaGestureEventArgs)
            If gestureInProgress Then
                Dim newDragFactor As Double = dragFactor - args.HorizontalChange / pageHosts(pageHostBaseIndex).ActualWidth

                If newDragFactor < 0 AndAlso (Not canGoPrevPage) AndAlso Me.ChapterIndex <> 0 AndAlso Me.PageIndex = 0 Then
                    pageHosts((pageHostBaseIndex + 2) Mod 3).Child = PageProvider.GetLastPage(prevPageChapterIndex, prevPagePageIndex)
                    canGoPrevPage = pageHosts((pageHostBaseIndex + 2) Mod 3).Child IsNot Nothing
                End If

                Dim maximum As Double = If(canGoNextPage, 1, 0)
                Dim minimum As Double = If(canGoPrevPage, -1, 0)
                dragFactor = Math.Max(minimum, Math.Min(maximum, newDragFactor))
                Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex + dragFactor
            End If
        End Sub

        Private Sub OnGestureListenerDragCompleted(ByVal sender As Object, ByVal args As DragCompletedGestureEventArgs)
            If gestureInProgress AndAlso (Not gestureHadFlick) Then
                If dragFactor > 0.5 Then
                    AnimateToNextPage()

                ElseIf dragFactor < -0.5 Then
                    AnimateToPreviousPage()

                Else
                    AnimateToSamePage()
                End If
            End If
        End Sub

        Private Sub SetUpNextPage()
            nextPageChapterIndex = Me.ChapterIndex
            nextPagePageIndex = Me.PageIndex + 1
            Dim element As FrameworkElement = Me.PageProvider.GetPage(nextPageChapterIndex, nextPagePageIndex)

            If element Is Nothing Then
                nextPageChapterIndex += 1
                nextPagePageIndex = 0
                element = Me.PageProvider.GetPage(nextPageChapterIndex, nextPagePageIndex)
            End If

            pageHosts((pageHostBaseIndex + 1) Mod 3).Child = element
        End Sub

        Private Sub SetUpPreviousPage()
            prevPageChapterIndex = Me.ChapterIndex
            prevPagePageIndex = Me.PageIndex - 1
            Dim element As FrameworkElement = Nothing

            If prevPagePageIndex <> -1 Then
                element = Me.PageProvider.GetPage(prevPageChapterIndex, prevPagePageIndex)
            ElseIf prevPageChapterIndex <> 0 Then
                prevPageChapterIndex -= 1
                element = Me.PageProvider.QueryLastPage(prevPageChapterIndex, prevPagePageIndex)
            End If
            pageHosts((pageHostBaseIndex + 2) Mod 3).Child = element
        End Sub

        Private Sub AnimateToNextPage()
            Dim anima As DoubleAnimation = New DoubleAnimation With {.To = pageHostBaseIndex + 1, .Duration = CalculateFractionalDuration(pageHostBaseIndex + 1)}

            Storyboard.SetTarget(anima, Me.PageTransition)
            Storyboard.SetTargetProperty(anima, New PropertyPath(PageTransition.FractionalBaseIndexProperty))

            Dim storyboardControl As New Storyboard()
            storyboardControl.Children.Add(anima)

            AddHandler storyboardControl.Completed, Sub(sender As Object, args As EventArgs)
                                                        pageHostBaseIndex = (pageHostBaseIndex + 1) Mod 3
                                                        Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex

                                                        ' Shift the indices from current to previous
                                                        prevPageChapterIndex = Me.ChapterIndex
                                                        prevPagePageIndex = Me.PageIndex

                                                        ' Shift the indices from next to current
                                                        Me.ChapterIndex = nextPageChapterIndex
                                                        Me.PageIndex = nextPagePageIndex

                                                        ' Set up the new next page
                                                        SetUpNextPage()
                                                        animationInProgress = False
                                                    End Sub

            animationInProgress = True
            storyboardControl.Begin()
        End Sub

        Private Sub AnimateToPreviousPage()
            Dim anima As DoubleAnimation = New DoubleAnimation With {.To = pageHostBaseIndex - 1, .Duration = CalculateFractionalDuration(pageHostBaseIndex - 1)}

            storyboard.SetTarget(anima, Me.PageTransition)
            Storyboard.SetTargetProperty(anima, New PropertyPath(PageTransition.FractionalBaseIndexProperty))

            Dim storyboardControl As New Storyboard()
            storyboardControl.Children.Add(anima)

            AddHandler storyboardControl.Completed, Sub(sender As Object, args As EventArgs)
                                                        pageHostBaseIndex = (pageHostBaseIndex + 2) Mod 3
                                                        Me.PageTransition.FractionalBaseIndex = pageHostBaseIndex

                                                        ' Transfer the indices from current to next
                                                        nextPageChapterIndex = Me.ChapterIndex
                                                        nextPagePageIndex = Me.PageIndex

                                                        ' Transfer the indices from previous to current
                                                        Me.ChapterIndex = prevPageChapterIndex
                                                        Me.PageIndex = prevPagePageIndex

                                                        ' Set up the new previous page
                                                        SetUpPreviousPage()
                                                        animationInProgress = False
                                                    End Sub

            animationInProgress = True
            storyboardControl.Begin()
        End Sub

        Private Sub AnimateToSamePage()
            Dim anima As DoubleAnimation = New DoubleAnimation With {.To = pageHostBaseIndex, .Duration = CalculateFractionalDuration(pageHostBaseIndex)}

            Storyboard.SetTarget(anima, Me.PageTransition)
            Storyboard.SetTargetProperty(anima, New PropertyPath(PageTransition.FractionalBaseIndexProperty))

            Dim storyboardControl As New Storyboard()
            storyboardControl.Children.Add(anima)

            AddHandler storyboardControl.Completed, Sub(sender As Object, args As EventArgs) animationInProgress = False

            animationInProgress = True
            storyboardControl.Begin()
        End Sub

        Private Function CalculateFractionalDuration(ByVal fractionalBaseIndexDestination As Double) As Duration
            Dim fraction As Double = Math.Abs(fractionalBaseIndexDestination - Me.PageTransition.FractionalBaseIndex)
            Dim msec As Double = fraction * Me.PageTransition.AnimationDuration
            Dim timeSpan As TimeSpan = TimeSpan.FromMilliseconds(msec)
            Return New Duration(timeSpan)
        End Function

        Private Shared Sub OnPageProviderChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
        End Sub

        Private Shared Sub OnPageTransitionChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
            TryCast(obj, BookViewer).OnPageTransitionChanged(CType(args.OldValue, PageTransition), CType(args.NewValue, PageTransition))
        End Sub

        Private Sub OnPageTransitionChanged(ByVal oldPageTransition As PageTransition, ByVal newPageTransition As PageTransition)
            Dim fractionalBaseIndex As Double = -1

            If oldPageTransition IsNot Nothing Then
                fractionalBaseIndex = oldPageTransition.FractionalBaseIndex
                oldPageTransition.Detach()
            End If
            If newPageTransition IsNot Nothing Then
                newPageTransition.Attach(LayoutRoot, pageContainer0, pageContainer1, pageContainer2)
                newPageTransition.FractionalBaseIndex = fractionalBaseIndex
            End If
        End Sub

        Private Shared Sub OnPageChanged(ByVal obj As DependencyObject, ByVal args As DependencyPropertyChangedEventArgs)
            Dim bookViewer As BookViewer = TryCast(obj, BookViewer)

            If bookViewer.PageChangedEvent IsNot Nothing Then
                bookViewer.PageChangedEvent(bookViewer, EventArgs.Empty)
            End If



        End Sub
    End Class
End Namespace