﻿Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media
Imports System.Windows.Shapes

Namespace MiddlemarchReader
    Public Class CurlTransition
        Inherits PageTransition
        Private Const curlFromTop As Boolean = True ' calculations really assume that bottom right corner curls
        Private Const R1 As Double = 1.0 / 3 ' portion of circle radius above bottom (or below top) of page as fraction of page height
        Private Const R As Double = (1 + R1 * R1) / (2 * R1) ' radius of arc governing the passage of the lower right (or upper right) corner

        Private isAttached As Boolean
        Private containerPanel As Panel
        Private pageContainers(2) As FrameworkElement

        ' Clipping geometry
        Private pathGeometry As New PathGeometry()
        Private pathFigure As PathFigure = New PathFigure With {.IsClosed = True}
        Private polylineSegment As New PolyLineSegment()
        Private clipGeometryOwner As Integer = -1

        ' Back of page polygon
        Private polygon As Polygon = New Polygon With {.Stroke = New SolidColorBrush(Colors.Black), .Fill = New SolidColorBrush(Color.FromArgb(255, 224, 224, 224)), .Visibility = Visibility.Collapsed}

        Public Overrides Sub Attach(ByVal containerPanel As Panel, ByVal pageContainer0 As FrameworkElement, ByVal pageContainer1 As FrameworkElement, ByVal pageContainer2 As FrameworkElement)
            Me.containerPanel = containerPanel
            pageContainers(0) = pageContainer0
            pageContainers(1) = pageContainer1
            pageContainers(2) = pageContainer2

            pathFigure.Segments.Add(polylineSegment)
            pathGeometry.Figures.Add(pathFigure)

            ' Keep Polygon on top of all page containers
            Canvas.SetZIndex(polygon, 4)
            containerPanel.Children.Add(polygon)

            isAttached = True
        End Sub

        Public Overrides Sub Detach()
            If isAttached Then
                For i As Integer = 0 To 2
                    pageContainers(i).Clip = Nothing
                Next i

                containerPanel.Children.Remove(polygon)
                isAttached = False
            End If
        End Sub

        Protected Overrides Sub ShowPageTransition(ByVal baseIndex As Integer, ByVal fraction As Double)
            Dim nextIndex As Integer = (baseIndex + 1) Mod 3
            Dim prevIndex As Integer = (baseIndex + 2) Mod 3
            Dim curlIndex As Integer = 0
            Dim angle As Double = Math.Asin(1 / R) ' Initial angle traveling along arc

            If Not isAttached Then
                Return
            End If

            If fraction >= 0 Then
                Canvas.SetZIndex(pageContainers(prevIndex), 1)
                Canvas.SetZIndex(pageContainers(nextIndex), 2)
                Canvas.SetZIndex(pageContainers(baseIndex), 3)
                curlIndex = baseIndex
                angle *= 1 - 2 * fraction
            Else ' fraction < 0
                Canvas.SetZIndex(pageContainers(nextIndex), 1)
                Canvas.SetZIndex(pageContainers(baseIndex), 2)
                Canvas.SetZIndex(pageContainers(prevIndex), 3)
                curlIndex = prevIndex
                angle *= -1 - 2 * fraction
            End If

            If fraction = 0 Then
                If clipGeometryOwner <> -1 Then
                    pageContainers(clipGeometryOwner).Clip = Nothing
                    clipGeometryOwner = -1
                End If
                polygon.Visibility = Visibility.Collapsed
            Else
                Dim width As Double = pageContainers(curlIndex).ActualWidth
                Dim height As Double = pageContainers(curlIndex).ActualHeight

                ' Calculate coordinates of corner traveling along arc
                Dim xArc As Double = width * R * Math.Sin(angle)
                Dim yArc As Double = height * (1 + (R - R1) - R * Math.Cos(angle))

                ' Apply Pythagorean theorems to get bottom and righ intersects
                Dim xBottom As Double = (width * width - xArc * xArc - Math.Pow(height - yArc, 2)) / (2 * (width - xArc))
                Dim yRight As Double = (height * height - yArc * yArc - Math.Pow(width - xArc, 2)) / (2 * (height - yArc))

                ' These two factors govern alter Y coordinates so the page curls from the top
                Dim minuend As Double = If(curlFromTop, height, 0)
                Dim multiplier As Double = If(curlFromTop, -1, 1)

                ' The clipping path
                polylineSegment.Points.Clear()
                pathFigure.StartPoint = New Point(0, minuend)

                ' The back of the page polygon
                polygon.Points.Clear()

                If Not Double.IsInfinity(yRight) Then
                    If yRight > 0 Then
                        polylineSegment.Points.Add(New Point(width, minuend))
                        polylineSegment.Points.Add(New Point(width, minuend + multiplier * yRight))

                        polygon.Points.Add(New Point(width, minuend + multiplier * yRight))
                    Else ' the corner of the page has gone above the top (or below the bottom)
                        ' Use similar triangles to calculate intersect on top of page
                        Dim xTop As Double = ((width - xBottom) * yRight) / (height - yRight) + width
                        polylineSegment.Points.Add(New Point(xTop, minuend))

                        ' Use a vector to calculate corner of page above the top
                        Dim vector As New Point(width - xArc, yRight - yArc)
                        Dim length As Double = Math.Sqrt(vector.X * vector.X + vector.Y * vector.Y)
                        Dim xCorner As Double = xArc + height * vector.X / length
                        Dim yCorner As Double = yArc + height * vector.Y / length

                        polygon.Points.Add(New Point(xTop, minuend))
                        polygon.Points.Add(New Point(xCorner, minuend + multiplier * yCorner))
                    End If

                    polylineSegment.Points.Add(New Point(xBottom, minuend + multiplier * height))
                    polylineSegment.Points.Add(New Point(0, minuend + multiplier * height))

                    If clipGeometryOwner <> -1 AndAlso clipGeometryOwner <> curlIndex Then
                        pageContainers(clipGeometryOwner).Clip = Nothing
                    End If

                    pageContainers(curlIndex).Clip = pathGeometry
                    clipGeometryOwner = curlIndex

                    polygon.Points.Add(New Point(xArc, minuend + multiplier * yArc))
                    polygon.Points.Add(New Point(xBottom, minuend + multiplier * height))
                    polygon.Visibility = Visibility.Visible
                End If
            End If
        End Sub
    End Class
End Namespace