﻿Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Media

Namespace MiddlemarchReader
    Public Class FlipTransition
        Inherits PageTransition
        Private pageContainers(2) As FrameworkElement
        Private projections(2) As PlaneProjection

        Public Overrides ReadOnly Property AnimationDuration() As Double
            Get
                Return 500
            End Get
        End Property

        Public Overrides Sub Attach(ByVal containerPanel As Panel, ByVal pageContainer0 As FrameworkElement, ByVal pageContainer1 As FrameworkElement, ByVal pageContainer2 As FrameworkElement)
            pageContainers(0) = pageContainer0
            pageContainers(1) = pageContainer1
            pageContainers(2) = pageContainer2

            For i As Integer = 0 To 2
                projections(i) = New PlaneProjection With {.CenterOfRotationX = 0}
                pageContainers(i).Projection = projections(i)
            Next i
        End Sub

        Public Overrides Sub Detach()
            For Each pageContainer As FrameworkElement In pageContainers
                pageContainer.RenderTransform = Nothing
            Next pageContainer
        End Sub

        Protected Overrides Sub ShowPageTransition(ByVal baseIndex As Integer, ByVal fraction As Double)
            Dim nextIndex As Integer = (baseIndex + 1) Mod 3
            Dim prevIndex As Integer = (baseIndex + 2) Mod 3

            If fraction >= 0 Then
                Canvas.SetZIndex(pageContainers(prevIndex), 1)
                Canvas.SetZIndex(pageContainers(nextIndex), 2)
                Canvas.SetZIndex(pageContainers(baseIndex), 3)

                projections(baseIndex).RotationY = 90 * fraction
                projections(nextIndex).RotationY = 0
                projections(prevIndex).RotationY = 0
            Else
                Canvas.SetZIndex(pageContainers(nextIndex), 1)
                Canvas.SetZIndex(pageContainers(baseIndex), 2)
                Canvas.SetZIndex(pageContainers(prevIndex), 3)

                projections(prevIndex).RotationY = 90 * (1 + fraction)
                projections(baseIndex).RotationY = 0
                projections(nextIndex).RotationY = 0
            End If
        End Sub
    End Class
End Namespace