﻿Imports System
Imports System.Windows
Imports System.Windows.Controls
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell

Namespace MiddlemarchReader
    Partial Public Class MainPage
        Inherits PhoneApplicationPage
        Private appSettings As AppSettings = (TryCast(Application.Current, App)).AppSettings
        Private book As BookInfo

        Public Sub New()
            InitializeComponent()

            ' Set the fields of the appbar buttons and menu
            chaptersIconButton = TryCast(Me.ApplicationBar.Buttons(0), ApplicationBarIconButton)
            slidingMenuItem = TryCast(Me.ApplicationBar.MenuItems(0), ApplicationBarMenuItem)
            curlingMenuItem = TryCast(Me.ApplicationBar.MenuItems(1), ApplicationBarMenuItem)
            flippingMenuItem = TryCast(Me.ApplicationBar.MenuItems(2), ApplicationBarMenuItem)

            Select Case appSettings.PageTransition
                Case "slide"
                    OnAppBarMenuItemClick(slidingMenuItem, EventArgs.Empty)
                Case "curl"
                    OnAppBarMenuItemClick(curlingMenuItem, EventArgs.Empty)
                Case "flip"
                    OnAppBarMenuItemClick(flippingMenuItem, EventArgs.Empty)
            End Select

            book = appSettings.Book

            AddHandler Loaded, Sub(sender As Object, args As RoutedEventArgs) bookViewer.JumpToPage(book.CurrentChapterIndex, book.CurrentPageIndex)
        End Sub

        Private Sub OnChaptersListBoxSelectionChanged(ByVal sender As Object, ByVal args As SelectionChangedEventArgs)
            Dim chapter As ChapterInfo = TryCast(chaptersListBox.SelectedItem, ChapterInfo)

            If chapter IsNot Nothing Then
                book.CurrentChapterIndex = book.Chapters.IndexOf(chapter)
                book.CurrentPageIndex = 0
                bookViewer.JumpToPage(book.CurrentChapterIndex, book.CurrentPageIndex)
            End If
            SwitchToBookView()
        End Sub

        Private Sub OnAppBarChaptersButtonClick(ByVal sender As Object, ByVal args As EventArgs)
            chaptersListBox.SelectedItem = Nothing

            If chaptersListBox.Visibility = Visibility.Visible Then
                SwitchToBookView()
            Else
                SwitchToChaptersView()
            End If
        End Sub

        Private Sub SwitchToBookView()
            chaptersIconButton.IconUri = New Uri("Images/chapters.icon.png", UriKind.Relative)
            chaptersIconButton.Text = "chapters"
            chaptersListBox.Visibility = Visibility.Collapsed
            bookViewer.Visibility = Visibility.Visible
        End Sub

        Private Sub SwitchToChaptersView()
            chaptersIconButton.IconUri = New Uri("Images/text.icon.png", UriKind.Relative)
            chaptersIconButton.Text = "text"
            chaptersListBox.Visibility = Visibility.Visible
            bookViewer.Visibility = Visibility.Collapsed
        End Sub

        Private Sub OnAppBarMenuItemClick(ByVal sender As Object, ByVal args As EventArgs)
            For Each menuItem As ApplicationBarMenuItem In Me.ApplicationBar.MenuItems
                Dim ch As Char = If(sender Is menuItem, ChrW(&H25C9), ChrW(&H25EF))
                menuItem.Text = AscW(ch) & menuItem.Text.Substring(1)
            Next menuItem

            If sender Is slidingMenuItem Then
                appSettings.PageTransition = "slide"
                bookViewer.PageTransition = New SlideTransition()

            ElseIf sender Is curlingMenuItem Then
                appSettings.PageTransition = "curl"
                bookViewer.PageTransition = New CurlTransition()

            ElseIf sender Is flippingMenuItem Then
                appSettings.PageTransition = "flip"
                bookViewer.PageTransition = New FlipTransition()
            End If
        End Sub

        Private Sub OnBookViewerPageChanged(ByVal sender As Object, ByVal args As EventArgs)
            Dim chapterIndex As Integer = bookViewer.ChapterIndex
            Dim pageIndex As Integer = bookViewer.PageIndex

            If chapterIndex < book.Chapters.Count AndAlso pageIndex < book.Chapters(chapterIndex).Pages.Count Then
                ' Save for isolated storage settings
                book.CurrentChapterIndex = bookViewer.ChapterIndex
                book.CurrentPageIndex = bookViewer.PageIndex

                ' Update the percentage indicator
                Dim accumulatedCharacterCount As Integer = book.Chapters(chapterIndex).Pages(pageIndex).AccumulatedCharacterCount
                Dim percentage As Double = 100.0 * accumulatedCharacterCount / book.CharacterCount
                percentageRun.Text = String.Format("{0:F1}%", percentage)
            End If
        End Sub
    End Class
End Namespace